@@ -4,7 +4,7 @@ use strict;
 use warnings;
 
 our $VERSION;
-$VERSION = '1.03';
+$VERSION = '1.04';
 
 =head1 NAME
 
@@ -15,13 +15,15 @@ Net::DNS::Resolver::Unbound - Unbound resolver base for Net::DNS
     use Net::DNS;
     use Net::DNS::Resolver::Unbound;
     my $resolver = Net::DNS::Resolver::Unbound->new(...);
+    my $response = $resolver->send(...);
 
 =head1 DESCRIPTION
 
-Net::DNS::Resolver::Unbound is designed as an extension to an existing Net::DNS installation.
+Net::DNS::Resolver::Unbound is designed as an extension to an existing
+Net::DNS installation which provides DNSSEC validated name resolution.
 
-Net::DNS::Resolver::Unbound replaces the resolver send() and bgsend() functionality in
-the Net::DNS::Resolver::Base implementation.
+Net::DNS::Resolver::Unbound replaces the resolver send() and bgsend()
+functionality in the Net::DNS::Resolver::Base implementation.
 
 As of this writing, the implementation has some significant limitations:
 
@@ -29,13 +31,19 @@ As of this writing, the implementation has some significant limitations:
 
 =item *
 
-Nameserver selection is entirely at the discretion of Unbound.
-Any user-specified nameservers are ignored.
+Selection of transport protocol and associated parameters is entirely
+at the discretion of Unbound.
 
 =item *
 
-It is not possible to send a pre-constructed DNS query packet to a nameserver.
-A best-effort attempt is instead made using the specified (qname,qtype,qclass).
+There is no provision for specifying DNS header flags or EDNS options
+in outbound packets.
+
+=item *
+
+It is not possible to send a pre-constructed DNS query packet to a
+nameserver. A best-effort attempt is made using (qname,qtype,qclass)
+extracted from the presented packet.
 
 =back
 
@@ -54,11 +62,22 @@ eval { Net::DNS::Resolver::Unbound->bootstrap($VERSION) } || croak $@;
 =head2 new
 
     my $resolver = Net::DNS::Resolver::Unbound->new(
-	debug  => 1,
+	debug	    => 1,
+	defnames    => 1,
+	dnsrch,	    => 1,
+	domain	    => 'domain',
+	ndots	    => 1,
+	searchlist  => ['domain' ... ],
+	nameservers => [ ... ],
 	option => ['logfile', 'mylog.txt'] );
 
 Returns a new Net::DNS::Resolver::Unbound resolver object.
 
+
+=head2 nameservers, force_v6, prefer_v6, force_v4, prefer_v4
+
+See L<Net::DNS::Resolver>.
+
 =cut
 
 sub new {
@@ -84,6 +103,7 @@ See L<Net::DNS::Resolver>.
 
 sub send {
 	my $self = shift;
+	$self->_finalise_config;
 	$self->_reset_errorstring;
 
 	my ($query) = $self->_make_query_packet(@_)->question;
@@ -93,8 +113,8 @@ sub send {
 
 	my $ub_ctx = $self->{ub_ctx};
 	my $result = eval {
-		bless( Net::DNS::Resolver::libunbound::ub_resolve( $ub_ctx, $qname, $qtype, $qclass ),
-			'Net::DNS::Resolver::Unbound::Result' );
+		my $ub_result = Net::DNS::Resolver::libunbound::ub_resolve( $ub_ctx, $qname, $qtype, $qclass );
+		bless( $ub_result, 'Net::DNS::Resolver::Unbound::Result' );
 	};
 	$self->errorstring($@);
 	return $self->_decode_result($result);
@@ -102,6 +122,7 @@ sub send {
 
 sub bgsend {
 	my $self = shift;
+	$self->_finalise_config;
 	$self->_reset_errorstring;
 
 	my ($query) = $self->_make_query_packet(@_)->question;
@@ -161,10 +182,128 @@ sub option {
 }
 
 
+=head2 config
+
+    $resolver->config( 'Unbound.cfg' );
+
+This is a power-users interface that lets you specify all sorts of
+Unbound configuration options.
+
+=cut
+
+sub config {
+	my ( $self, $filename ) = @_;
+	my $ctx = $self->{ub_ctx};
+	return Net::DNS::Resolver::libunbound::ub_ctx_config( $ctx, $filename );
+}
+
+
+=head2 set_fwd
+
+    $resolver->set_fwd( 'IP address' );
+
+Set IPv4 or IPv6 address to which DNS queries are to be directed.
+The destination machine is expected to run a recursive resolver.
+If the proxy is not DNSSEC-capable, validation may fail.
+Can be called several times, in that case the addresses are used
+as backup servers.
+
+
+=cut
+
+sub set_fwd {
+	my ( $self, $address ) = @_;
+	my $ctx = $self->{ub_ctx};
+	return Net::DNS::Resolver::libunbound::ub_ctx_set_fwd( $ctx, $address || return );
+}
+
+
+=head2 set_tls
+
+    $resolver->set_tls( 0 );
+    $resolver->set_tls( 1 );
+
+Use DNS over TLS to send queries to machines specified using set_fwd().
+
+=cut
+
+sub set_tls {
+	my ( $self, $boolean ) = @_;
+	my $ctx = $self->{ub_ctx};
+	return Net::DNS::Resolver::libunbound::ub_ctx_set_tls( $ctx, $boolean );
+}
+
+
+=head2 set_stub
+
+    $resolver->set_stub( 'zone', '10.1.2.3', 0 );
+
+Add a stub zone, with given address to send to. This is for custom root
+hints or pointing to a local authoritative dns server. For dns resolvers
+and the 'DHCP DNS' ip address, use ub_ctx_set_fwd.
+
+=cut
+
+sub set_stub {
+	my ( $self, $zone, $address, $prime ) = @_;
+	my $ctx = $self->{ub_ctx};
+	return Net::DNS::Resolver::libunbound::ub_ctx_set_stub( $ctx, $zone, $address, $prime );
+}
+
+
+=head2 add_ta
+
+    $resolver->add_ta( 'trust anchor' );
+
+Add a trust anchor which is a string that holds a valid DNSKEY or DS RR
+in RFC1035 zonefile format.
+
+=cut
+
+sub add_ta {
+	my ( $self, $trust_anchor ) = @_;
+	my $ctx = $self->{ub_ctx};
+	my $ta	= Net::DNS::RR->new($trust_anchor);
+	return Net::DNS::Resolver::libunbound::ub_ctx_add_ta( $ctx, $ta->plain );
+}
+
+
+=head2 add_ta_file
+
+    $resolver->add_ta_file( 'filename' );
+
+Pass the name of a file containing DS and DNSKEY records (like from dig
+or drill).
+
+=cut
+
+sub add_ta_file {
+	my ( $self, $filename ) = @_;
+	my $ctx = $self->{ub_ctx};
+	return Net::DNS::Resolver::libunbound::ub_ctx_add_ta_file( $ctx, $filename );
+}
+
+=head2 trustedkeys
+
+    $resolver->trustedkeys( 'filename' );
+
+Pass the name of a bind-style config file comtaining trusted-keys{}.
+
+=cut
+
+sub trustedkeys {
+	my ( $self, $filename ) = @_;
+	my $ctx = $self->{ub_ctx};
+	return Net::DNS::Resolver::libunbound::ub_ctx_trustedkeys( $ctx, $filename );
+}
+
+
 =head2 debug_level
 
-Set verbosity of the debug output directed to stderr.
-Level 0 is off, 1 very minimal, 2 detailed, and 3 lots.
+    $resolver->debug_level(0);
+
+Set verbosity of the debug output directed to stderr.  Level 0 is off,
+1 very minimal, 2 detailed, and 3 lots.
 
 =cut
 
@@ -178,8 +317,11 @@ sub debug_level {
 
 =head2 async_thread
 
-Enable a call to resolve_async() to create a thread to handle work in the background.
-If false (by default), a process is forked to handle work in the background.
+    $resolver->async_thread(1);
+
+Enable a call to resolve_async() to create a thread to handle work in
+the background. If false (by default), a process is forked to handle
+work in the background.
 
 =cut
 
@@ -194,6 +336,28 @@ sub async_thread {
 
 sub replyfrom { return "(local) Unbound resolver" }
 
+sub string {
+	my $self = shift;
+
+	$self = $self->_defaults unless ref($self);
+
+	my @nslist   = $self->nameservers();
+	my ($force)  = ( grep( { $self->{$_} } qw(force_v6 force_v4) ),	  'force_v4' );
+	my ($prefer) = ( grep( { $self->{$_} } qw(prefer_v6 prefer_v4) ), 'prefer_v4' );
+	return <<END;
+;; RESOLVER state:
+;; nameservers	= @nslist
+;; searchlist	= @{$self->{searchlist}}
+;; defnames	= $self->{defnames}	dnsrch		= $self->{dnsrch}
+;; ${prefer}	= $self->{$prefer}	${force}	= $self->{$force}
+;; debug	= $self->{debug}	ndots		= $self->{ndots}
+END
+}
+
+sub print {
+	return print shift->string;
+}
+
 
 sub _decode_result {
 	my ( $self, $result ) = @_;
@@ -210,6 +374,17 @@ sub _decode_result {
 }
 
 
+sub _finalise_config {
+	my $self = shift;
+	return if $self->{ub_frozen}++;
+	my @nameservers = $self->nameservers;
+	$self->set_fwd( shift @nameservers );
+	$self->set_fwd( shift @nameservers );
+	$self->set_fwd( shift @nameservers );
+	return;
+}
+
+
 1;
 __END__
 
